@@ -3,28 +3,28 @@ import uuid
 from datetime import datetime, timedelta
 
 import OpenSSL
+import swapper
 from django.core.exceptions import ValidationError
 from django.db import models
 from django.utils import timezone
-from django.utils.encoding import python_2_unicode_compatible
 from django.utils.functional import cached_property
+from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext_lazy as _
 from jsonfield import JSONField
 from model_utils.fields import AutoCreatedField, AutoLastModifiedField
 from OpenSSL import crypto
-from six import string_types
 
 from .. import settings as app_settings
-from ..utils import bytes_compat
 
 generalized_time = '%Y%m%d%H%M%SZ'
+utc_time = '%y%m%d%H%M%SZ'
 
 KEY_LENGTH_CHOICES = (
     ('', ''),
     ('512', '512'),
     ('1024', '1024'),
     ('2048', '2048'),
-    ('4096', '4096')
+    ('4096', '4096'),
 )
 
 DIGEST_CHOICES = (
@@ -45,6 +45,17 @@ SIGNATURE_MAPPING = {
 }
 
 
+def datetime_to_string(datetime_):
+    """
+    Converts datetime.datetime object to UTCTime/GeneralizedTime string
+    following RFC5280. (Returns string encoded in UTCtime for dates through year
+    2049, otherwise in GeneralizedTime format)
+    """
+    if datetime_.year < 2050:
+        return datetime_.strftime(utc_time)
+    return datetime_.strftime(generalized_time)
+
+
 def default_validity_start():
     """
     Sets validity_start field to 1 day before the current date
@@ -93,56 +104,74 @@ def default_digest_algorithm():
     return app_settings.DEFAULT_DIGEST_ALGORITHM
 
 
-@python_2_unicode_compatible
 class BaseX509(models.Model):
     """
     Abstract Cert class, shared between Ca and Cert
     """
+
     name = models.CharField(max_length=64)
     notes = models.TextField(blank=True)
-    key_length = models.CharField(_('key length'),
-                                  help_text=_('bits'),
-                                  blank=True,
-                                  choices=KEY_LENGTH_CHOICES,
-                                  default=default_key_length,
-                                  max_length=6)
-    digest = models.CharField(_('digest algorithm'),
-                              help_text=_('bits'),
-                              blank=True,
-                              choices=DIGEST_CHOICES,
-                              default=default_digest_algorithm,
-                              max_length=8)
-    validity_start = models.DateTimeField(blank=True,
-                                          null=True,
-                                          default=default_validity_start)
-    validity_end = models.DateTimeField(blank=True,
-                                        null=True,
-                                        default=default_cert_validity_end)
+    key_length = models.CharField(
+        _('key length'),
+        help_text=_('bits'),
+        blank=True,
+        choices=KEY_LENGTH_CHOICES,
+        default=default_key_length,
+        max_length=6,
+    )
+    digest = models.CharField(
+        _('digest algorithm'),
+        help_text=_('bits'),
+        blank=True,
+        choices=DIGEST_CHOICES,
+        default=default_digest_algorithm,
+        max_length=8,
+    )
+    validity_start = models.DateTimeField(
+        blank=True, null=True, default=default_validity_start
+    )
+    validity_end = models.DateTimeField(
+        blank=True, null=True, default=default_cert_validity_end
+    )
     country_code = models.CharField(max_length=2, blank=True)
     state = models.CharField(_('state or province'), max_length=64, blank=True)
     city = models.CharField(_('city'), max_length=64, blank=True)
     organization_name = models.CharField(_('organization'), max_length=64, blank=True)
-    organizational_unit_name = models.CharField(_('organizational unit name'),
-                                                max_length=64, blank=True)
+    organizational_unit_name = models.CharField(
+        _('organizational unit name'), max_length=64, blank=True
+    )
     email = models.EmailField(_('email address'), blank=True)
-    common_name = models.CharField(_('common name'), max_length=63, blank=True)
-    extensions = JSONField(_('extensions'),
-                           default=list,
-                           blank=True,
-                           help_text=_('additional x509 certificate extensions'),
-                           load_kwargs={'object_pairs_hook': collections.OrderedDict},
-                           dump_kwargs={'indent': 4})
+    common_name = models.CharField(_('common name'), max_length=64, blank=True)
+    extensions = JSONField(
+        _('extensions'),
+        default=list,
+        blank=True,
+        help_text=_('additional x509 certificate extensions'),
+        load_kwargs={'object_pairs_hook': collections.OrderedDict},
+        dump_kwargs={'indent': 4},
+    )
     # serial_number is set to CharField as a UUID integer is too big for a
     # PositiveIntegerField and an IntegerField on SQLite
-    serial_number = models.CharField(_('serial number'),
-                                     help_text=_('leave blank to determine automatically'),
-                                     blank=True,
-                                     null=True,
-                                     max_length=39)
-    certificate = models.TextField(blank=True, help_text='certificate in X.509 PEM format')
-    private_key = models.TextField(blank=True, help_text='private key in X.509 PEM format')
+    serial_number = models.CharField(
+        _('serial number'),
+        help_text=_('leave blank to determine automatically'),
+        blank=True,
+        null=True,
+        max_length=48,
+    )
+    certificate = models.TextField(
+        blank=True, help_text='certificate in X.509 PEM format'
+    )
+    private_key = models.TextField(
+        blank=True, help_text='private key in X.509 PEM format'
+    )
     created = AutoCreatedField(_('created'), editable=True)
     modified = AutoLastModifiedField(_('modified'), editable=True)
+    passphrase = models.CharField(
+        max_length=64,
+        blank=True,
+        help_text=_('Passphrase for the private key, if present'),
+    )
 
     class Meta:
         abstract = True
@@ -157,32 +186,35 @@ class BaseX509(models.Model):
         if self._state.adding and self.certificate and self.private_key:
             self._validate_pem()
             self._import()
-        super(BaseX509, self).clean_fields(*args, **kwargs)
+        super().clean_fields(*args, **kwargs)
 
     def clean(self):
         # when importing, both public and private must be present
-        if (
-            (self.certificate and not self.private_key) or
-            (self.private_key and not self.certificate)
+        if (self.certificate and not self.private_key) or (
+            self.private_key and not self.certificate
         ):
-            raise ValidationError(_('When importing an existing certificate, both'
-                                    'keys (private and public) must be present'))
+            raise ValidationError(
+                _(
+                    'When importing an existing certificate, both'
+                    'keys (private and public) must be present'
+                )
+            )
         if self.serial_number:
             self._validate_serial_number()
         self._verify_extension_format()
 
     def save(self, *args, **kwargs):
         generate = False
-        if not self.id and not self.certificate and not self.private_key:
+        if not self.pk and not self.certificate and not self.private_key:
             generate = True
-        super(BaseX509, self).save(*args, **kwargs)
+        super().save(*args, **kwargs)
         if generate:
             # automatically determine serial number
             if not self.serial_number:
-                self.serial_number = uuid.uuid4().int
+                self.serial_number = self._generate_serial_number()
             self._generate()
             kwargs['force_insert'] = False
-            super(BaseX509, self).save(*args, **kwargs)
+            super().save(*args, **kwargs)
 
     @cached_property
     def x509(self):
@@ -208,7 +240,11 @@ class BaseX509(models.Model):
         returns an instance of OpenSSL.crypto.PKey
         """
         if self.private_key:
-            return crypto.load_privatekey(crypto.FILETYPE_PEM, self.private_key)
+            return crypto.load_privatekey(
+                crypto.FILETYPE_PEM,
+                self.private_key,
+                passphrase=getattr(self, 'passphrase').encode('utf-8'),
+            )
 
     def _validate_pem(self):
         """
@@ -220,9 +256,20 @@ class BaseX509(models.Model):
             method_name = 'load_{0}'.format(field.replace('_', ''))
             load_pem = getattr(crypto, method_name)
             try:
-                load_pem(crypto.FILETYPE_PEM, getattr(self, field))
+                args = (crypto.FILETYPE_PEM, getattr(self, field))
+                kwargs = {}
+                if method_name == 'load_privatekey':
+                    kwargs['passphrase'] = getattr(self, 'passphrase').encode('utf8')
+                load_pem(*args, **kwargs)
             except OpenSSL.crypto.Error as e:
-                errors[field] = ValidationError(_('OpenSSL error: {0}'.format(e.args[0])))
+                error = 'OpenSSL error: <br>{0}'.format(
+                    str(e.args[0]).replace('), ', '), <br>').strip('[]')
+                )
+                if 'bad decrypt' in error:
+                    error = '<b>Incorrect Passphrase</b> <br>' + error
+                    errors['passphrase'] = ValidationError(_(mark_safe(error)))
+                    continue
+                errors[field] = ValidationError(_(mark_safe(error)))
         if errors:
             raise ValidationError(errors)
 
@@ -234,7 +281,9 @@ class BaseX509(models.Model):
         try:
             int(self.serial_number)
         except ValueError:
-            raise ValidationError({'serial_number': _('Serial number must be an integer')})
+            raise ValidationError(
+                {'serial_number': _('Serial number must be an integer')}
+            )
 
     def _generate(self):
         """
@@ -248,8 +297,8 @@ class BaseX509(models.Model):
         cert.set_version(0x2)  # version 3 (0 indexed counting)
         cert.set_subject(subject)
         cert.set_serial_number(int(self.serial_number))
-        cert.set_notBefore(bytes_compat(self.validity_start.strftime(generalized_time)))
-        cert.set_notAfter(bytes_compat(self.validity_end.strftime(generalized_time)))
+        cert.set_notBefore(bytes(str(datetime_to_string(self.validity_start)), 'utf8'))
+        cert.set_notAfter(bytes(str(datetime_to_string(self.validity_end)), 'utf8'))
         # generating certificate for CA
         if not hasattr(self, 'ca'):
             issuer = cert.get_subject()
@@ -262,8 +311,17 @@ class BaseX509(models.Model):
         cert.set_pubkey(key)
         cert = self._add_extensions(cert)
         cert.sign(issuer_key, str(self.digest))
-        self.certificate = crypto.dump_certificate(crypto.FILETYPE_PEM, cert).decode("utf-8")
-        self.private_key = crypto.dump_privatekey(crypto.FILETYPE_PEM, key).decode("utf-8")
+        self.certificate = crypto.dump_certificate(crypto.FILETYPE_PEM, cert).decode(
+            'utf-8'
+        )
+        key_args = (crypto.FILETYPE_PEM, key)
+        key_kwargs = {}
+        if self.passphrase:
+            key_kwargs['passphrase'] = self.passphrase.encode('utf-8')
+            key_kwargs['cipher'] = 'DES-EDE3-CBC'
+        self.private_key = crypto.dump_privatekey(*key_args, **key_kwargs).decode(
+            'utf-8'
+        )
 
     def _fill_subject(self, subject):
         """
@@ -277,7 +335,7 @@ class BaseX509(models.Model):
             'organization_name': 'organizationName',
             'organizational_unit_name': 'organizationalUnitName',
             'email': 'emailAddress',
-            'common_name': 'commonName'
+            'common_name': 'commonName',
         }
         # set x509 subject attributes only if not empty strings
         for model_attr, subject_attr in attr_map.items():
@@ -285,9 +343,7 @@ class BaseX509(models.Model):
             if value:
                 # coerce value to string, allow these fields to be redefined
                 # as foreign keys by subclasses without losing compatibility
-                if not isinstance(value, string_types):
-                    value = str(value)
-                setattr(subject, subject_attr, value)
+                setattr(subject, subject_attr, str(value))
         return subject
 
     def _import(self):
@@ -305,12 +361,10 @@ class BaseX509(models.Model):
         algorithm = cert.get_signature_algorithm().decode('utf8')
         self.digest = SIGNATURE_MAPPING[algorithm]
         not_before = cert.get_notBefore().decode('utf8')
-        self.validity_start = datetime.strptime(not_before,
-                                                generalized_time)
+        self.validity_start = datetime.strptime(not_before, generalized_time)
         self.validity_start = timezone.make_aware(self.validity_start)
         not_after = cert.get_notAfter().decode('utf8')
-        self.validity_end = datetime.strptime(not_after,
-                                              generalized_time)
+        self.validity_end = datetime.strptime(not_after, generalized_time)
         self.validity_end.replace(tzinfo=timezone.tzinfo())
         self.validity_end = timezone.make_aware(self.validity_end)
         subject = cert.get_subject()
@@ -340,8 +394,10 @@ class BaseX509(models.Model):
         try:
             store_ctx.verify_certificate()
         except crypto.X509StoreContextError as e:
-            raise ValidationError(_("CA doesn't match, got the "
-                                    "following error from pyOpenSSL: \"%s\"") % e.args[0][2])
+            raise ValidationError(
+                _("CA doesn't match, got the " 'following error from pyOpenSSL: "%s"')
+                % e.args[0][2]
+            )
 
     def _verify_extension_format(self):
         """
@@ -369,48 +425,75 @@ class BaseX509(models.Model):
             ext_value = 'CA:TRUE'
             if pathlen is not None:
                 ext_value = '{0}, pathlen:{1}'.format(ext_value, pathlen)
-            ext.append(crypto.X509Extension(b'basicConstraints',
-                                            app_settings.CA_BASIC_CONSTRAINTS_CRITICAL,
-                                            bytes_compat(ext_value)))
-            ext.append(crypto.X509Extension(b'keyUsage',
-                                            app_settings.CA_KEYUSAGE_CRITICAL,
-                                            bytes_compat(app_settings.CA_KEYUSAGE_VALUE)))
+            ext.append(
+                crypto.X509Extension(
+                    b'basicConstraints',
+                    app_settings.CA_BASIC_CONSTRAINTS_CRITICAL,
+                    bytes(str(ext_value), 'utf8'),
+                )
+            )
+            ext.append(
+                crypto.X509Extension(
+                    b'keyUsage',
+                    app_settings.CA_KEYUSAGE_CRITICAL,
+                    bytes(str(app_settings.CA_KEYUSAGE_VALUE), 'utf8'),
+                )
+            )
             issuer_cert = cert
         # prepare extensions for end-entity certs
         else:
-            ext.append(crypto.X509Extension(b'basicConstraints',
-                                            False,
-                                            b'CA:FALSE'))
-            ext.append(crypto.X509Extension(b'keyUsage',
-                                            app_settings.CERT_KEYUSAGE_CRITICAL,
-                                            bytes_compat(app_settings.CERT_KEYUSAGE_VALUE)))
+            ext.append(crypto.X509Extension(b'basicConstraints', False, b'CA:FALSE'))
+            ext.append(
+                crypto.X509Extension(
+                    b'keyUsage',
+                    app_settings.CERT_KEYUSAGE_CRITICAL,
+                    bytes(str(app_settings.CERT_KEYUSAGE_VALUE), 'utf8'),
+                )
+            )
             issuer_cert = self.ca.x509
-        ext.append(crypto.X509Extension(b'subjectKeyIdentifier',
-                                        False,
-                                        b'hash',
-                                        subject=cert))
+        ext.append(
+            crypto.X509Extension(b'subjectKeyIdentifier', False, b'hash', subject=cert)
+        )
         cert.add_extensions(ext)
         # authorityKeyIdentifier must be added after
         # the other extensions have been already added
-        cert.add_extensions([
-            crypto.X509Extension(b'authorityKeyIdentifier',
-                                 False,
-                                 b'keyid:always,issuer:always',
-                                 issuer=issuer_cert)
-        ])
+        cert.add_extensions(
+            [
+                crypto.X509Extension(
+                    b'authorityKeyIdentifier',
+                    False,
+                    b'keyid:always,issuer:always',
+                    issuer=issuer_cert,
+                )
+            ]
+        )
         for ext in self.extensions:
-            cert.add_extensions([
-                crypto.X509Extension(bytes_compat(ext['name']),
-                                     bool(ext['critical']),
-                                     bytes_compat(ext['value']))
-            ])
+            cert.add_extensions(
+                [
+                    crypto.X509Extension(
+                        bytes(str(ext['name']), 'utf8'),
+                        bool(ext['critical']),
+                        bytes(str(ext['value']), 'utf8'),
+                    )
+                ]
+            )
         return cert
 
+    def renew(self):
+        self._generate()
+        self.serial_number = self._generate_serial_number()
+        self.validity_end = self.__class__().validity_end
+        self.save()
+
+    def _generate_serial_number(self):
+        return uuid.uuid4().int
+
 
 class AbstractCa(BaseX509):
     """
     Abstract Ca model
     """
+
     class Meta:
         abstract = True
         verbose_name = _('CA')
@@ -422,9 +505,18 @@ class AbstractCa(BaseX509):
         (does not include expired certificates)
         """
         now = timezone.now()
-        return self.cert_set.filter(revoked=True,
-                                    validity_start__lte=now,
-                                    validity_end__gte=now)
+        return self.cert_set.filter(
+            revoked=True, validity_start__lte=now, validity_end__gte=now
+        )
+
+    def renew(self):
+        """
+        Renew the certificate, private key and
+        validity end date of a CA
+        """
+        super().renew()
+        for cert in self.cert_set.all():
+            cert.renew()
 
     @property
     def crl(self):
@@ -433,12 +525,12 @@ class AbstractCa(BaseX509):
         """
         revoked_certs = self.get_revoked_certs()
         crl = crypto.CRL()
-        now_str = timezone.now().strftime(generalized_time)
+        now_str = datetime_to_string(timezone.now())
         for cert in revoked_certs:
             revoked = crypto.Revoked()
-            revoked.set_serial(bytes_compat(cert.serial_number))
+            revoked.set_serial(bytes(str(cert.serial_number), 'utf8'))
             revoked.set_reason(b'unspecified')
-            revoked.set_rev_date(bytes_compat(now_str))
+            revoked.set_rev_date(bytes(str(now_str), 'utf8'))
             crl.add_revoked(revoked)
         return crl.export(self.x509, self.pkey, days=1, digest=b'sha256')
 
@@ -450,13 +542,16 @@ class AbstractCert(BaseX509):
     """
     Abstract Cert model
     """
-    ca = models.ForeignKey('django_x509.Ca', on_delete=models.CASCADE, verbose_name=_('CA'))
-    revoked = models.BooleanField(_('revoked'),
-                                  default=False)
-    revoked_at = models.DateTimeField(_('revoked at'),
-                                      blank=True,
-                                      null=True,
-                                      default=None)
+
+    ca = models.ForeignKey(
+        swapper.get_model_name('django_x509', 'Ca'),
+        on_delete=models.CASCADE,
+        verbose_name=_('CA'),
+    )
+    revoked = models.BooleanField(_('revoked'), default=False)
+    revoked_at = models.DateTimeField(
+        _('revoked at'), blank=True, null=True, default=None
+    )
 
     def __str__(self):
         return self.name
