@@ -1,3 +1,5 @@
+import swapper
+
 from .base.models import AbstractCa, AbstractCert
 
 
@@ -5,13 +7,17 @@ class Ca(AbstractCa):
     """
     Concrete Ca model
     """
+
     class Meta(AbstractCa.Meta):
         abstract = False
+        swappable = swapper.swappable_setting('django_x509', 'Ca')
 
 
 class Cert(AbstractCert):
     """
     Concrete Cert model
     """
+
     class Meta(AbstractCert.Meta):
         abstract = False
+        swappable = swapper.swappable_setting('django_x509', 'Cert')
