@@ -5,19 +5,20 @@ from django.test import TestCase
 from django.urls import reverse
 from django.utils import timezone
 from OpenSSL import crypto
+from swapper import load_model
 
-from . import TestX509Mixin
 from .. import settings as app_settings
-from ..base.models import generalized_time
-from ..models import Ca, Cert
+from ..base.models import datetime_to_string, generalized_time, utc_time
+from . import TestX509Mixin
+
+Ca = load_model('django_x509', 'Ca')
+Cert = load_model('django_x509', 'Cert')
 
 
 class TestCa(TestX509Mixin, TestCase):
     """
     tests for Ca model
     """
-    ca_model = Ca
-    cert_model = Cert
 
     def _prepare_revoked(self):
         ca = self._create_ca()
@@ -139,9 +140,13 @@ WRyKPvMvJzWT
         # verify field attribtues
         self.assertEqual(ca.key_length, '512')
         self.assertEqual(ca.digest, 'sha1')
-        start = timezone.make_aware(datetime.strptime('20150101000000Z', generalized_time))
+        start = timezone.make_aware(
+            datetime.strptime('20150101000000Z', generalized_time)
+        )
         self.assertEqual(ca.validity_start, start)
-        end = timezone.make_aware(datetime.strptime('20200101000000Z', generalized_time))
+        end = timezone.make_aware(
+            datetime.strptime('20200101000000Z', generalized_time)
+        )
         self.assertEqual(ca.validity_end, end)
         self.assertEqual(ca.country_code, 'US')
         self.assertEqual(ca.state, 'CA')
@@ -155,8 +160,9 @@ WRyKPvMvJzWT
         self.assertEqual(cert.get_version(), 3)
         ca.delete()
         # test auto name
-        ca = Ca(certificate=self.import_certificate,
-                private_key=self.import_private_key)
+        ca = Ca(
+            certificate=self.import_certificate, private_key=self.import_private_key
+        )
         ca.full_clean()
         ca.save()
         self.assertEqual(ca.name, 'importtest')
@@ -221,10 +227,9 @@ WRyKPvMvJzWT
         e = ca.x509.get_extension(2)
         self.assertEqual(e.get_short_name().decode(), 'subjectKeyIdentifier')
         self.assertEqual(e.get_critical(), False)
-        e2 = crypto.X509Extension(b'subjectKeyIdentifier',
-                                  False,
-                                  b'hash',
-                                  subject=ca.x509)
+        e2 = crypto.X509Extension(
+            b'subjectKeyIdentifier', False, b'hash', subject=ca.x509
+        )
         self.assertEqual(e.get_data(), e2.get_data())
 
     def test_authority_key_identifier(self):
@@ -232,18 +237,20 @@ WRyKPvMvJzWT
         e = ca.x509.get_extension(3)
         self.assertEqual(e.get_short_name().decode(), 'authorityKeyIdentifier')
         self.assertEqual(e.get_critical(), False)
-        e2 = crypto.X509Extension(b'authorityKeyIdentifier',
-                                  False,
-                                  b'keyid:always,issuer:always',
-                                  issuer=ca.x509)
+        e2 = crypto.X509Extension(
+            b'authorityKeyIdentifier',
+            False,
+            b'keyid:always,issuer:always',
+            issuer=ca.x509,
+        )
         self.assertEqual(e.get_data(), e2.get_data())
 
     def test_extensions(self):
         extensions = [
             {
-                "name": "nsComment",
-                "critical": False,
-                "value": "CA - autogenerated Certificate"
+                'name': 'nsComment',
+                'critical': False,
+                'value': 'CA - autogenerated Certificate',
             }
         ]
         ca = self._create_ca(extensions=extensions)
@@ -263,7 +270,7 @@ WRyKPvMvJzWT
             self.fail('ValidationError not raised')
 
     def test_extensions_error2(self):
-        extensions = [{"wrong": "wrong"}]
+        extensions = [{'wrong': 'wrong'}]
         try:
             self._create_ca(extensions=extensions)
         except ValidationError as e:
@@ -276,7 +283,7 @@ WRyKPvMvJzWT
         ca = self._create_ca()
         c1 = self._create_cert(ca=ca)
         c2 = self._create_cert(ca=ca)
-        c3 = self._create_cert(ca=ca)  # noqa
+        self._create_cert(ca=ca)
         self.assertEqual(ca.get_revoked_certs().count(), 0)
         c1.revoke()
         self.assertEqual(ca.get_revoked_certs().count(), 1)
@@ -286,17 +293,13 @@ WRyKPvMvJzWT
         # expired certificates are not counted
         start = now - timedelta(days=6650)
         end = now - timedelta(days=6600)
-        c4 = self._create_cert(ca=ca,
-                               validity_start=start,
-                               validity_end=end)
+        c4 = self._create_cert(ca=ca, validity_start=start, validity_end=end)
         c4.revoke()
         self.assertEqual(ca.get_revoked_certs().count(), 2)
         # inactive not counted yet
         start = now + timedelta(days=2)
         end = now + timedelta(days=365)
-        c5 = self._create_cert(ca=ca,
-                               validity_start=start,
-                               validity_end=end)
+        c5 = self._create_cert(ca=ca, validity_start=start, validity_end=end)
         c5.revoke()
         self.assertEqual(ca.get_revoked_certs().count(), 2)
 
@@ -310,7 +313,7 @@ WRyKPvMvJzWT
 
     def test_crl_view(self):
         ca, cert = self._prepare_revoked()
-        response = self.client.get(reverse('x509:crl', args=[ca.pk]))
+        response = self.client.get(reverse('admin:crl', args=[ca.pk]))
         self.assertEqual(response.status_code, 200)
         crl = crypto.load_crl(crypto.FILETYPE_PEM, response.content)
         revoked_list = crl.get_revoked()
@@ -320,11 +323,16 @@ WRyKPvMvJzWT
 
     def test_crl_view_403(self):
         setattr(app_settings, 'CRL_PROTECTED', True)
-        ca, cert = self._prepare_revoked()
-        response = self.client.get(reverse('x509:crl', args=[ca.pk]))
+        ca, _ = self._prepare_revoked()
+        response = self.client.get(reverse('admin:crl', args=[ca.pk]))
         self.assertEqual(response.status_code, 403)
         setattr(app_settings, 'CRL_PROTECTED', False)
 
+    def test_crl_view_404(self):
+        self._prepare_revoked()
+        response = self.client.get(reverse('admin:crl', args=[10]))
+        self.assertEqual(response.status_code, 404)
+
     def test_x509_text(self):
         ca = self._create_ca()
         text = crypto.dump_certificate(crypto.FILETYPE_TEXT, ca.x509)
@@ -453,37 +461,51 @@ wZWuZRQLPPTAdiW+drs3gz8w0u3Y9ihgvHQqFcGJ1+j6ANJ0XdE/D5Y=
 -----END RSA PRIVATE KEY-----"""
 
     def test_ca_invalid_country(self):
-        ca = self._create_ca(name='ImportTest error',
-                             certificate=self.problematic_certificate,
-                             private_key=self.problematic_private_key)
+        ca = self._create_ca(
+            name='ImportTest error',
+            certificate=self.problematic_certificate,
+            private_key=self.problematic_private_key,
+        )
         self.assertEqual(ca.country_code, '')
 
     def test_import_ca_cert_validation_error(self):
         certificate = self.import_certificate[20:]
         private_key = self.import_private_key
-        ca = Ca(name="TestCaCertValidation")
+        ca = Ca(name='TestCaCertValidation')
         try:
             ca.certificate = certificate
             ca.private_key = private_key
             ca.full_clean()
         except ValidationError as e:
-            self.assertIn("[('PEM routines', 'PEM_read_bio', 'no start line')]",
-                          str(e.message_dict['certificate'][0]))
+            # cryptography 2.4 and 2.6 have different error message formats
+            error_msg = str(e.message_dict['certificate'][0])
+            self.assertTrue(
+                "('PEM routines', 'PEM_read_bio', 'no start line')"
+                in error_msg  # cryptography 2.4+
+                or "('PEM routines', 'get_name', 'no start line')"
+                in error_msg  # cryptography 2.6+
+            )
         else:
             self.fail('ValidationError not raised')
 
     def test_import_ca_key_validation_error(self):
         certificate = self.import_certificate
         private_key = self.import_private_key[20:]
-        ca = Ca(name="TestCaKeyValidation")
+        ca = Ca(name='TestCaKeyValidation')
         try:
             ca.certificate = certificate
             ca.private_key = private_key
             ca.full_clean()
             ca.save()
         except ValidationError as e:
-            self.assertIn("[('PEM routines', 'PEM_read_bio', 'no start line')]",
-                          str(e.message_dict['private_key'][0]))
+            # cryptography 2.4 and 2.6 have different error message formats
+            error_msg = str(e.message_dict['private_key'][0])
+            self.assertTrue(
+                "('PEM routines', 'PEM_read_bio', 'no start line')"
+                in error_msg  # cryptography 2.4+
+                or "('PEM routines', 'get_name', 'no start line')"
+                in error_msg  # cryptography 2.6+
+            )
         else:
             self.fail('ValidationError not raised')
 
@@ -497,4 +519,165 @@ wZWuZRQLPPTAdiW+drs3gz8w0u3Y9ihgvHQqFcGJ1+j6ANJ0XdE/D5Y=
         try:
             self._create_ca(serial_number='notIntegers')
         except ValidationError as e:
-            self.assertEqual("Serial number must be an integer", str(e.message_dict['serial_number'][0]))
+            self.assertEqual(
+                'Serial number must be an integer',
+                str(e.message_dict['serial_number'][0]),
+            )
+
+    def test_import_ca_key_with_passphrase(self):
+        ca = Ca(name='ImportTest')
+        ca.certificate = """-----BEGIN CERTIFICATE-----
+MIICrzCCAhigAwIBAgIJANCybYj5LwUWMA0GCSqGSIb3DQEBCwUAMG8xCzAJBgNV
+BAYTAklOMQwwCgYDVQQIDANhc2QxDDAKBgNVBAcMA2FzZDEMMAoGA1UECgwDYXNk
+MQwwCgYDVQQLDANhc2QxDDAKBgNVBAMMA2FzZDEaMBgGCSqGSIb3DQEJARYLYXNk
+QGFzZC5hc2QwHhcNMTgwODI5MjExMDQ1WhcNMTkwODI5MjExMDQ1WjBvMQswCQYD
+VQQGEwJJTjEMMAoGA1UECAwDYXNkMQwwCgYDVQQHDANhc2QxDDAKBgNVBAoMA2Fz
+ZDEMMAoGA1UECwwDYXNkMQwwCgYDVQQDDANhc2QxGjAYBgkqhkiG9w0BCQEWC2Fz
+ZEBhc2QuYXNkMIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQDBuDdlU20Ydie8
+tmbq2hn8Ski6aSH2IyVVMxUj3+i6QZmoJ4sZzcAMCLPIkCAxby5pP0V6/DSqjxTL
+ShYy/7QMCovmj3O+23eYR/JGNAfsk6uDsWJL6OLHTNdx19mL0NioeFNEUJt14Cbz
+uqUizT7UdONLer0UK4uP2sE09Eo4cQIDAQABo1MwUTAdBgNVHQ4EFgQURUEc1+ho
+on8xaoSU+HU6CRkn0/owHwYDVR0jBBgwFoAURUEc1+hoon8xaoSU+HU6CRkn0/ow
+DwYDVR0TAQH/BAUwAwEB/zANBgkqhkiG9w0BAQsFAAOBgQB2zU8qtkXVM25yrL9s
+FC5oSqTky2c9KI/hwdsSronSvwaMoASgfl7UjzXlovq9FWZpNSVZ06wetkJVjq5N
+Xn3APftPSmKw0J1tzUfZuvq8Z8q6uXQ4B2+BsiCkG/PwXizbKDc29yzXsXTL4+cQ
+J7RrWKwDUi/GKVvqc+JjgsQ/nA==
+-----END CERTIFICATE-----
+
+        """
+        ca.private_key = """-----BEGIN RSA PRIVATE KEY-----
+Proc-Type: 4,ENCRYPTED
+DEK-Info: DES-EDE3-CBC,D7DDAD38C7462384
+
+CUEPD7buBQqv/uipFz/tXYURNcQrY5HKU904IVsKbM233KPA6qU6IaRF6RRxxUtE
+ejrmY2es9ZmU63gO/G/16E0CxzWhm3G2pOBsWHsBGGYcMpqZ842E3NoWimfQuRyO
+E7TtMKW+Jdl6mzkw8s/KkSeGkGvZFKrclSN37CtkexRn4cXQkhNgPztyeRaQjIBM
+SveP2qbODU+lr8g2oUjx05Ftcv1zJin85tzifJlQyaQz8ozKYtHA/RSpLEFZ19HG
+mXn4Rvvai8r2zhdqfT/0/G6dABDrhQLxQhPE2MrY0hAlr7DnDrYNQQ/QyGoiAdcR
+ee7QUDNfDnjzU6k/EjYPU1827/Kw8R4al8yDtVcUqfDuEsKabot+krEx4IZ5LOk9
+PkcSW8UR0cIm7QE2BzQEzaZKQIpVwjSsSKm+RcFktiCKVun3Sps+GtXBr+AmF5Na
+r6xeg+j9kz8lT8F5lnpFTk6c8cD8GDCRiLsFzPo652BQ24dAEPvsSbYmKwr1gEe8
+tfsARqOuvSafQNzqBYFV7abFr8DFiE1Kghk6d6x2u7qVREvOh0RYHRWqsTRf4MMn
+WlEnL9zfYST9Ur3gJgBOH2WHboDlQZu1k7yoLMfiGTQSQ2/xg1zS+5IWxt4tg029
+B+f39N5zyDjuGFYcf3J6J4zybHmvdSAa62qxnkeDIbLz4axTU8+hNNOWxIsAh5vs
+OO8quCk6DE4j4u3Yzk7810dkJtliwboQiTlitEbCjiyjkOrabIICKMte8nhylZX6
+BxZA3knyYRiB0FNYSxI6YuCIqTjr0AoBvNHdkdjkv2VFomYNBd8ruA==
+-----END RSA PRIVATE KEY-----
+        """
+        ca.passphrase = 'test123'
+        ca.full_clean()
+        ca.save()
+        self.assertIsInstance(ca.pkey, crypto.PKey)
+
+    def test_import_ca_key_with_incorrect_passphrase(self):
+        ca = Ca(name='ImportTest')
+        ca.certificate = """-----BEGIN CERTIFICATE-----
+MIICrzCCAhigAwIBAgIJANCybYj5LwUWMA0GCSqGSIb3DQEBCwUAMG8xCzAJBgNV
+BAYTAklOMQwwCgYDVQQIDANhc2QxDDAKBgNVBAcMA2FzZDEMMAoGA1UECgwDYXNk
+MQwwCgYDVQQLDANhc2QxDDAKBgNVBAMMA2FzZDEaMBgGCSqGSIb3DQEJARYLYXNk
+QGFzZC5hc2QwHhcNMTgwODI5MjExMDQ1WhcNMTkwODI5MjExMDQ1WjBvMQswCQYD
+VQQGEwJJTjEMMAoGA1UECAwDYXNkMQwwCgYDVQQHDANhc2QxDDAKBgNVBAoMA2Fz
+ZDEMMAoGA1UECwwDYXNkMQwwCgYDVQQDDANhc2QxGjAYBgkqhkiG9w0BCQEWC2Fz
+ZEBhc2QuYXNkMIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQDBuDdlU20Ydie8
+tmbq2hn8Ski6aSH2IyVVMxUj3+i6QZmoJ4sZzcAMCLPIkCAxby5pP0V6/DSqjxTL
+ShYy/7QMCovmj3O+23eYR/JGNAfsk6uDsWJL6OLHTNdx19mL0NioeFNEUJt14Cbz
+uqUizT7UdONLer0UK4uP2sE09Eo4cQIDAQABo1MwUTAdBgNVHQ4EFgQURUEc1+ho
+on8xaoSU+HU6CRkn0/owHwYDVR0jBBgwFoAURUEc1+hoon8xaoSU+HU6CRkn0/ow
+DwYDVR0TAQH/BAUwAwEB/zANBgkqhkiG9w0BAQsFAAOBgQB2zU8qtkXVM25yrL9s
+FC5oSqTky2c9KI/hwdsSronSvwaMoASgfl7UjzXlovq9FWZpNSVZ06wetkJVjq5N
+Xn3APftPSmKw0J1tzUfZuvq8Z8q6uXQ4B2+BsiCkG/PwXizbKDc29yzXsXTL4+cQ
+J7RrWKwDUi/GKVvqc+JjgsQ/nA==
+-----END CERTIFICATE-----
+
+        """
+        ca.private_key = """-----BEGIN RSA PRIVATE KEY-----
+Proc-Type: 4,ENCRYPTED
+DEK-Info: DES-EDE3-CBC,D7DDAD38C7462384
+
+CUEPD7buBQqv/uipFz/tXYURNcQrY5HKU904IVsKbM233KPA6qU6IaRF6RRxxUtE
+ejrmY2es9ZmU63gO/G/16E0CxzWhm3G2pOBsWHsBGGYcMpqZ842E3NoWimfQuRyO
+E7TtMKW+Jdl6mzkw8s/KkSeGkGvZFKrclSN37CtkexRn4cXQkhNgPztyeRaQjIBM
+SveP2qbODU+lr8g2oUjx05Ftcv1zJin85tzifJlQyaQz8ozKYtHA/RSpLEFZ19HG
+mXn4Rvvai8r2zhdqfT/0/G6dABDrhQLxQhPE2MrY0hAlr7DnDrYNQQ/QyGoiAdcR
+ee7QUDNfDnjzU6k/EjYPU1827/Kw8R4al8yDtVcUqfDuEsKabot+krEx4IZ5LOk9
+PkcSW8UR0cIm7QE2BzQEzaZKQIpVwjSsSKm+RcFktiCKVun3Sps+GtXBr+AmF5Na
+r6xeg+j9kz8lT8F5lnpFTk6c8cD8GDCRiLsFzPo652BQ24dAEPvsSbYmKwr1gEe8
+tfsARqOuvSafQNzqBYFV7abFr8DFiE1Kghk6d6x2u7qVREvOh0RYHRWqsTRf4MMn
+WlEnL9zfYST9Ur3gJgBOH2WHboDlQZu1k7yoLMfiGTQSQ2/xg1zS+5IWxt4tg029
+B+f39N5zyDjuGFYcf3J6J4zybHmvdSAa62qxnkeDIbLz4axTU8+hNNOWxIsAh5vs
+OO8quCk6DE4j4u3Yzk7810dkJtliwboQiTlitEbCjiyjkOrabIICKMte8nhylZX6
+BxZA3knyYRiB0FNYSxI6YuCIqTjr0AoBvNHdkdjkv2VFomYNBd8ruA==
+-----END RSA PRIVATE KEY-----
+        """
+        try:
+            ca.passphrase = 'incorrect_passphrase'
+            ca.full_clean()
+            ca.save()
+        except ValidationError as e:
+            self.assertIn('Incorrect Passphrase', str(e.message_dict['passphrase'][0]))
+        else:
+            self.fail('ValidationError not raised')
+
+    def test_generate_ca_with_passphrase(self):
+        ca = self._create_ca(passphrase='123')
+        ca.full_clean()
+        ca.save()
+        self.assertIsInstance(ca.pkey, crypto.PKey)
+
+    def test_datetime_to_string(self):
+        generalized_datetime = datetime(2050, 1, 1, 0, 0, 0, 0)
+        utc_datetime = datetime(2049, 12, 31, 0, 0, 0, 0)
+        self.assertEqual(
+            datetime_to_string(generalized_datetime),
+            generalized_datetime.strftime(generalized_time),
+        )
+        self.assertEqual(
+            datetime_to_string(utc_datetime), utc_datetime.strftime(utc_time)
+        )
+
+    def test_renew(self):
+        ca = self._create_ca()
+        cert1 = self._create_cert(ca=ca, name='cert1')
+        cert2 = self._create_cert(ca=ca, name='cert2')
+        old_ca_cert = ca.certificate
+        old_ca_key = ca.private_key
+        old_ca_end = ca.validity_end
+        old_ca_serial_number = ca.serial_number
+        old_cert1_cert = cert1.certificate
+        old_cert1_key = cert1.private_key
+        old_cert1_serial_number = cert1.serial_number
+        old_cert1_end = cert1.validity_end
+        old_cert2_cert = cert2.certificate
+        old_cert2_key = cert2.private_key
+        old_cert2_serial_number = cert2.serial_number
+        old_cert2_end = cert2.validity_end
+        ca.renew()
+        cert1.refresh_from_db()
+        cert2.refresh_from_db()
+        self.assertNotEqual(old_ca_cert, ca.certificate)
+        self.assertNotEqual(old_ca_key, ca.private_key)
+        self.assertLess(old_ca_end, ca.validity_end)
+        self.assertNotEqual(old_ca_serial_number, ca.serial_number)
+        self.assertNotEqual(old_cert1_cert, cert1.certificate)
+        self.assertNotEqual(old_cert1_key, cert1.private_key)
+        self.assertLess(old_cert1_end, cert1.validity_end)
+        self.assertNotEqual(old_cert1_serial_number, cert1.serial_number)
+        self.assertNotEqual(old_cert2_cert, cert2.certificate)
+        self.assertNotEqual(old_cert2_key, cert2.private_key)
+        self.assertLess(old_cert2_end, cert2.validity_end)
+        self.assertNotEqual(old_cert2_serial_number, cert2.serial_number)
+
+    def test_ca_common_name_length(self):
+        common_name = (
+            'this is a very very very very very very'
+            ' very very very very very very long name'
+        )
+        with self.assertRaises(ValidationError) as context_manager:
+            self._create_ca(common_name=common_name)
+
+        msg = (
+            f'Ensure this value has at most 64 characters (it has {len(common_name)}).'
+        )
+        message_dict = context_manager.exception.message_dict
+        self.assertIn('common_name', message_dict)
+        self.assertEqual(message_dict['common_name'][0], msg)
