@@ -4,19 +4,21 @@ from django.core.exceptions import ValidationError
 from django.test import TestCase
 from django.utils import timezone
 from OpenSSL import crypto
+from swapper import load_model
 
-from . import TestX509Mixin
 from .. import settings as app_settings
 from ..base.models import generalized_time
-from ..models import Ca, Cert
+from . import TestX509Mixin
+
+Ca = load_model('django_x509', 'Ca')
+Cert = load_model('django_x509', 'Cert')
 
 
 class TestCert(TestX509Mixin, TestCase):
     """
     tests for Cert model
     """
-    ca_model = Ca
-    cert_model = Cert
+
     import_certificate = """
 -----BEGIN CERTIFICATE-----
 MIICMTCCAdugAwIBAgIDAeJAMA0GCSqGSIb3DQEBBQUAMGgxETAPBgNVBAoMCE9w
@@ -150,10 +152,12 @@ k9Y1S1C9VB0YsDZTeZUggJNSDN4YrKjIevYZQQIhAOWec6vngM/PlI1adrFndd3d
         ca.private_key = self.import_ca_private_key
         ca.full_clean()
         ca.save()
-        cert = Cert(name='ImportCertTest',
-                    ca=ca,
-                    certificate=self.import_certificate,
-                    private_key=self.import_private_key)
+        cert = Cert(
+            name='ImportCertTest',
+            ca=ca,
+            certificate=self.import_certificate,
+            private_key=self.import_private_key,
+        )
         cert.full_clean()
         cert.save()
         x509 = cert.x509
@@ -176,9 +180,13 @@ k9Y1S1C9VB0YsDZTeZUggJNSDN4YrKjIevYZQQIhAOWec6vngM/PlI1adrFndd3d
         # verify field attribtues
         self.assertEqual(cert.key_length, '512')
         self.assertEqual(cert.digest, 'sha1')
-        start = timezone.make_aware(datetime.strptime('20151101000000Z', generalized_time))
+        start = timezone.make_aware(
+            datetime.strptime('20151101000000Z', generalized_time)
+        )
         self.assertEqual(cert.validity_start, start)
-        end = timezone.make_aware(datetime.strptime('21181102180025Z', generalized_time))
+        end = timezone.make_aware(
+            datetime.strptime('21181102180025Z', generalized_time)
+        )
         self.assertEqual(cert.validity_end, end)
         self.assertEqual(cert.country_code, '')
         self.assertEqual(cert.state, '')
@@ -191,9 +199,11 @@ k9Y1S1C9VB0YsDZTeZUggJNSDN4YrKjIevYZQQIhAOWec6vngM/PlI1adrFndd3d
         self.assertEqual(x509.get_version(), 2)
         cert.delete()
         # test auto name
-        cert = Cert(certificate=self.import_certificate,
-                    private_key=self.import_private_key,
-                    ca=ca)
+        cert = Cert(
+            certificate=self.import_certificate,
+            private_key=self.import_private_key,
+            ca=ca,
+        )
         cert.full_clean()
         cert.save()
         self.assertEqual(cert.name, '123456')
@@ -204,8 +214,7 @@ k9Y1S1C9VB0YsDZTeZUggJNSDN4YrKjIevYZQQIhAOWec6vngM/PlI1adrFndd3d
         ca.private_key = self.import_ca_private_key
         ca.full_clean()
         ca.save()
-        cert = Cert(name='ImportTest',
-                    ca=ca)
+        cert = Cert(name='ImportTest', ca=ca)
         cert.certificate = self.import_certificate
         try:
             cert.full_clean()
@@ -217,14 +226,16 @@ k9Y1S1C9VB0YsDZTeZUggJNSDN4YrKjIevYZQQIhAOWec6vngM/PlI1adrFndd3d
 
     def test_import_wrong_ca(self):
         # test auto name
-        cert = Cert(certificate=self.import_certificate,
-                    private_key=self.import_private_key,
-                    ca=self._create_ca())
+        cert = Cert(
+            certificate=self.import_certificate,
+            private_key=self.import_private_key,
+            ca=self._create_ca(),
+        )
         try:
             cert.full_clean()
         except ValidationError as e:
             # verify error message
-            self.assertIn('CA doesn\'t match', str(e.message_dict['__all__'][0]))
+            self.assertIn("CA doesn't match", str(e.message_dict['__all__'][0]))
         else:
             self.fail('ValidationError not raised')
 
@@ -249,17 +260,18 @@ k9Y1S1C9VB0YsDZTeZUggJNSDN4YrKjIevYZQQIhAOWec6vngM/PlI1adrFndd3d
         e = cert.x509.get_extension(1)
         self.assertEqual(e.get_short_name().decode(), 'keyUsage')
         self.assertEqual(e.get_data(), b'\x03\x02\x07\x80')
-        setattr(app_settings, 'CERT_KEYUSAGE_VALUE', 'digitalSignature, keyEncipherment')
+        setattr(
+            app_settings, 'CERT_KEYUSAGE_VALUE', 'digitalSignature, keyEncipherment'
+        )
 
     def test_subject_key_identifier(self):
         cert = self._create_cert()
         e = cert.x509.get_extension(2)
         self.assertEqual(e.get_short_name().decode(), 'subjectKeyIdentifier')
         self.assertEqual(e.get_critical(), False)
-        e2 = crypto.X509Extension(b'subjectKeyIdentifier',
-                                  False,
-                                  b'hash',
-                                  subject=cert.x509)
+        e2 = crypto.X509Extension(
+            b'subjectKeyIdentifier', False, b'hash', subject=cert.x509
+        )
         self.assertEqual(e.get_data(), e2.get_data())
 
     def test_authority_key_identifier(self):
@@ -267,24 +279,22 @@ k9Y1S1C9VB0YsDZTeZUggJNSDN4YrKjIevYZQQIhAOWec6vngM/PlI1adrFndd3d
         e = cert.x509.get_extension(3)
         self.assertEqual(e.get_short_name().decode(), 'authorityKeyIdentifier')
         self.assertEqual(e.get_critical(), False)
-        e2 = crypto.X509Extension(b'authorityKeyIdentifier',
-                                  False,
-                                  b'keyid:always,issuer:always',
-                                  issuer=cert.ca.x509)
+        e2 = crypto.X509Extension(
+            b'authorityKeyIdentifier',
+            False,
+            b'keyid:always,issuer:always',
+            issuer=cert.ca.x509,
+        )
         self.assertEqual(e.get_data(), e2.get_data())
 
     def test_extensions(self):
         extensions = [
+            {'name': 'nsCertType', 'critical': False, 'value': 'client'},
             {
-                "name": "nsCertType",
-                "critical": False,
-                "value": "client"
+                'name': 'extendedKeyUsage',
+                'critical': True,  # critical just for testing purposes
+                'value': 'clientAuth',
             },
-            {
-                "name": "extendedKeyUsage",
-                "critical": True,  # critical just for testing purposes
-                "value": "clientAuth"
-            }
         ]
         cert = self._create_cert(extensions=extensions)
         e1 = cert.x509.get_extension(4)
@@ -307,9 +317,7 @@ k9Y1S1C9VB0YsDZTeZUggJNSDN4YrKjIevYZQQIhAOWec6vngM/PlI1adrFndd3d
             self.fail('ValidationError not raised')
 
     def test_extensions_error2(self):
-        extensions = [
-            {"wrong": "wrong"}
-        ]
+        extensions = [{'wrong': 'wrong'}]
         try:
             self._create_cert(extensions=extensions)
         except ValidationError as e:
@@ -352,11 +360,7 @@ k9Y1S1C9VB0YsDZTeZUggJNSDN4YrKjIevYZQQIhAOWec6vngM/PlI1adrFndd3d
         ca.full_clean()
         ca.save()
 
-        Cert.objects.create(
-            ca=ca,
-            common_name='TestCert1',
-            name='TestCert1',
-        )
+        Cert.objects.create(ca=ca, common_name='TestCert1', name='TestCert1')
 
     def test_import_cert_validation_error(self):
         certificate = self.import_certificate[20:]
@@ -367,14 +371,22 @@ k9Y1S1C9VB0YsDZTeZUggJNSDN4YrKjIevYZQQIhAOWec6vngM/PlI1adrFndd3d
         ca.full_clean()
         ca.save()
         try:
-            cert = Cert(name='TestCertValidation',
-                        ca=ca,
-                        certificate=certificate,
-                        private_key=private_key)
+            cert = Cert(
+                name='TestCertValidation',
+                ca=ca,
+                certificate=certificate,
+                private_key=private_key,
+            )
             cert.full_clean()
         except ValidationError as e:
-            self.assertIn("[('PEM routines', 'PEM_read_bio', 'no start line')]",
-                          str(e.message_dict['certificate'][0]))
+            # cryptography 2.4 and 2.6 have different error message formats
+            error_msg = str(e.message_dict['certificate'][0])
+            self.assertTrue(
+                "('PEM routines', 'PEM_read_bio', 'no start line')"
+                in error_msg  # cryptography 2.4+
+                or "('PEM routines', 'get_name', 'no start line')"
+                in error_msg  # cryptography 2.6+
+            )
         else:
             self.fail('ValidationError not raised')
 
@@ -387,14 +399,22 @@ k9Y1S1C9VB0YsDZTeZUggJNSDN4YrKjIevYZQQIhAOWec6vngM/PlI1adrFndd3d
         ca.full_clean()
         ca.save()
         try:
-            cert = Cert(name='TestKeyValidation',
-                        ca=ca,
-                        certificate=certificate,
-                        private_key=private_key)
+            cert = Cert(
+                name='TestKeyValidation',
+                ca=ca,
+                certificate=certificate,
+                private_key=private_key,
+            )
             cert.full_clean()
         except ValidationError as e:
-            self.assertIn("[('PEM routines', 'PEM_read_bio', 'no start line')]",
-                          str(e.message_dict['private_key'][0]))
+            # cryptography 2.4 and 2.6 have different error message formats
+            error_msg = str(e.message_dict['private_key'][0])
+            self.assertTrue(
+                "('PEM routines', 'PEM_read_bio', 'no start line')"
+                in error_msg  # cryptography 2.4+
+                or "('PEM routines', 'get_name', 'no start line')"
+                in error_msg  # cryptography 2.6+
+            )
         else:
             self.fail('ValidationError not raised')
 
@@ -408,7 +428,10 @@ k9Y1S1C9VB0YsDZTeZUggJNSDN4YrKjIevYZQQIhAOWec6vngM/PlI1adrFndd3d
         try:
             self._create_cert(serial_number='notIntegers')
         except ValidationError as e:
-            self.assertEqual("Serial number must be an integer", str(e.message_dict['serial_number'][0]))
+            self.assertEqual(
+                'Serial number must be an integer',
+                str(e.message_dict['serial_number'][0]),
+            )
 
     def test_serial_number_clash(self):
         ca = Ca(name='TestSerialClash')
@@ -418,12 +441,102 @@ k9Y1S1C9VB0YsDZTeZUggJNSDN4YrKjIevYZQQIhAOWec6vngM/PlI1adrFndd3d
         cert = self._create_cert(serial_number=123456, ca=ca)
         cert.full_clean()
         cert.save()
-        _cert = Cert(name='TestClash',
-                     ca=ca,
-                     certificate=self.import_certificate,
-                     private_key=self.import_private_key)
+        _cert = Cert(
+            name='TestClash',
+            ca=ca,
+            certificate=self.import_certificate,
+            private_key=self.import_private_key,
+        )
         try:
             _cert.full_clean()
         except ValidationError as e:
-            self.assertEqual("Certificate with this CA and Serial number already exists.",
-                             str(e.message_dict['__all__'][0]))
+            self.assertEqual(
+                'Certificate with this CA and Serial number already exists.',
+                str(e.message_dict['__all__'][0]),
+            )
+
+    def test_import_cert_with_passphrase(self):
+        ca = Ca(name='ImportTest')
+        ca.certificate = """-----BEGIN CERTIFICATE-----
+MIICrzCCAhigAwIBAgIJANCybYj5LwUWMA0GCSqGSIb3DQEBCwUAMG8xCzAJBgNV
+BAYTAklOMQwwCgYDVQQIDANhc2QxDDAKBgNVBAcMA2FzZDEMMAoGA1UECgwDYXNk
+MQwwCgYDVQQLDANhc2QxDDAKBgNVBAMMA2FzZDEaMBgGCSqGSIb3DQEJARYLYXNk
+QGFzZC5hc2QwHhcNMTgwODI5MjExMDQ1WhcNMTkwODI5MjExMDQ1WjBvMQswCQYD
+VQQGEwJJTjEMMAoGA1UECAwDYXNkMQwwCgYDVQQHDANhc2QxDDAKBgNVBAoMA2Fz
+ZDEMMAoGA1UECwwDYXNkMQwwCgYDVQQDDANhc2QxGjAYBgkqhkiG9w0BCQEWC2Fz
+ZEBhc2QuYXNkMIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQDBuDdlU20Ydie8
+tmbq2hn8Ski6aSH2IyVVMxUj3+i6QZmoJ4sZzcAMCLPIkCAxby5pP0V6/DSqjxTL
+ShYy/7QMCovmj3O+23eYR/JGNAfsk6uDsWJL6OLHTNdx19mL0NioeFNEUJt14Cbz
+uqUizT7UdONLer0UK4uP2sE09Eo4cQIDAQABo1MwUTAdBgNVHQ4EFgQURUEc1+ho
+on8xaoSU+HU6CRkn0/owHwYDVR0jBBgwFoAURUEc1+hoon8xaoSU+HU6CRkn0/ow
+DwYDVR0TAQH/BAUwAwEB/zANBgkqhkiG9w0BAQsFAAOBgQB2zU8qtkXVM25yrL9s
+FC5oSqTky2c9KI/hwdsSronSvwaMoASgfl7UjzXlovq9FWZpNSVZ06wetkJVjq5N
+Xn3APftPSmKw0J1tzUfZuvq8Z8q6uXQ4B2+BsiCkG/PwXizbKDc29yzXsXTL4+cQ
+J7RrWKwDUi/GKVvqc+JjgsQ/nA==
+-----END CERTIFICATE-----
+
+        """
+        ca.private_key = """-----BEGIN RSA PRIVATE KEY-----
+Proc-Type: 4,ENCRYPTED
+DEK-Info: DES-EDE3-CBC,D7DDAD38C7462384
+
+CUEPD7buBQqv/uipFz/tXYURNcQrY5HKU904IVsKbM233KPA6qU6IaRF6RRxxUtE
+ejrmY2es9ZmU63gO/G/16E0CxzWhm3G2pOBsWHsBGGYcMpqZ842E3NoWimfQuRyO
+E7TtMKW+Jdl6mzkw8s/KkSeGkGvZFKrclSN37CtkexRn4cXQkhNgPztyeRaQjIBM
+SveP2qbODU+lr8g2oUjx05Ftcv1zJin85tzifJlQyaQz8ozKYtHA/RSpLEFZ19HG
+mXn4Rvvai8r2zhdqfT/0/G6dABDrhQLxQhPE2MrY0hAlr7DnDrYNQQ/QyGoiAdcR
+ee7QUDNfDnjzU6k/EjYPU1827/Kw8R4al8yDtVcUqfDuEsKabot+krEx4IZ5LOk9
+PkcSW8UR0cIm7QE2BzQEzaZKQIpVwjSsSKm+RcFktiCKVun3Sps+GtXBr+AmF5Na
+r6xeg+j9kz8lT8F5lnpFTk6c8cD8GDCRiLsFzPo652BQ24dAEPvsSbYmKwr1gEe8
+tfsARqOuvSafQNzqBYFV7abFr8DFiE1Kghk6d6x2u7qVREvOh0RYHRWqsTRf4MMn
+WlEnL9zfYST9Ur3gJgBOH2WHboDlQZu1k7yoLMfiGTQSQ2/xg1zS+5IWxt4tg029
+B+f39N5zyDjuGFYcf3J6J4zybHmvdSAa62qxnkeDIbLz4axTU8+hNNOWxIsAh5vs
+OO8quCk6DE4j4u3Yzk7810dkJtliwboQiTlitEbCjiyjkOrabIICKMte8nhylZX6
+BxZA3knyYRiB0FNYSxI6YuCIqTjr0AoBvNHdkdjkv2VFomYNBd8ruA==
+-----END RSA PRIVATE KEY-----
+        """
+        ca.passphrase = 'test123'
+        ca.full_clean()
+        ca.save()
+        self.assertIsInstance(ca.pkey, crypto.PKey)
+
+    def test_generate_ca_with_passphrase(self):
+        ca = self._create_ca(passphrase='123')
+        ca.full_clean()
+        ca.save()
+        self.assertIsInstance(ca.pkey, crypto.PKey)
+
+    def test_renew(self):
+        cert = self._create_cert()
+        old_cert = cert.certificate
+        old_key = cert.private_key
+        old_end = cert.validity_end
+        old_serial_number = cert.serial_number
+        old_ca_cert = cert.ca.certificate
+        old_ca_key = cert.ca.private_key
+        old_ca_end = cert.ca.validity_end
+        old_ca_serial_number = cert.ca.serial_number
+        cert.renew()
+        self.assertNotEqual(old_cert, cert.certificate)
+        self.assertNotEqual(old_key, cert.private_key)
+        self.assertLess(old_end, cert.validity_end)
+        self.assertNotEqual(old_serial_number, cert.serial_number)
+        self.assertEqual(old_ca_cert, cert.ca.certificate)
+        self.assertEqual(old_ca_key, cert.ca.private_key)
+        self.assertEqual(old_ca_end, cert.ca.validity_end)
+        self.assertEqual(old_ca_serial_number, cert.ca.serial_number)
+
+    def test_cert_common_name_length(self):
+        common_name = (
+            'this is a very very very very very very'
+            ' very very very very very very long name'
+        )
+        with self.assertRaises(ValidationError) as context_manager:
+            self._create_cert(common_name=common_name)
+
+        msg = (
+            f'Ensure this value has at most 64 characters (it has {len(common_name)}).'
+        )
+        message_dict = context_manager.exception.message_dict
+        self.assertIn('common_name', message_dict)
+        self.assertEqual(message_dict['common_name'][0], msg)
